'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runInNewWindow = exports.series = exports.concurrent = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _anyShellEscape = require('any-shell-escape');

var _anyShellEscape2 = _interopRequireDefault(_anyShellEscape);

var _isWindows = require('is-windows');

var _isWindows2 = _interopRequireDefault(_isWindows);

var _commonTags = require('common-tags');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultColors = ['bgBlue.bold', 'bgMagenta.bold', 'bgGreen.bold', 'bgBlack.bold', 'bgCyan.bold', 'bgRed.bold', 'bgWhite.bold', 'bgYellow.bold'];

exports.concurrent = concurrent;
exports.series = series;
exports.runInNewWindow = runInNewWindow;

/**
 * Accepts any number of scripts, filters out any
 * falsy ones and joins them with ' && '
 * @param {...string} scripts - Any number of strings representing commands
 * @example
 * // returns 'eslint && jest && webpack --env.production'
 * series('eslint', 'jest', 'webpack --env.production')
 * @return {string} - the command that will execute the given scripts in series
 */

function series() {
  for (var _len = arguments.length, scripts = Array(_len), _key = 0; _key < _len; _key++) {
    scripts[_key] = arguments[_key];
  }

  return scripts.filter(Boolean).join(' && ');
}

/**
 * Accepts any number of nps script names, filters out
 * any falsy ones, prepends `nps` to them, and passes
 * the that to `series`
 * @param {...string} scriptNames - the script names to run
 * // returns 'nps lint && nps "test --coverage" && nps build'
 * series.nps('lint', 'test --coverage', 'build')
 * @return {string} - the command that will execute the nps scripts in series
 */
series.nps = function seriesNPS() {
  for (var _len2 = arguments.length, scriptNames = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    scriptNames[_key2] = arguments[_key2];
  }

  return series.apply(undefined, _toConsumableArray(scriptNames.filter(Boolean).map(function (scriptName) {
    return scriptName.trim();
  }).filter(Boolean).map(function (scriptName) {
    return `nps ${quoteScript(scriptName)}`;
  })));
};

/**
 * A concurrent script object
 * @typedef {Object|string} ConcurrentScript
 * @property {string} script - the command to run
 * @property {string} color - the color to use
 *   (see concurrently's docs for valid values)
 */
/**
 * An object of concurrent script objects
 * @typedef {Object.<ConcurrentScript>} ConcurrentScripts
 */

/**
 * Generates a command that uses `concurrently` to run
 * scripts concurrently. Adds a few flags to make it
 * behave as you probably want (like --kill-others-on-fail).
 * In addition, it adds color and labels where the color
 * can be specified or is defaulted and the label is based
 * on the key for the script.
 * @param {ConcurrentScripts} scripts - the scripts to run
 *   note: this function filters out falsy values :)
 * @example
 * // returns a bit of a long script that can vary slightly
 * // based on your environment... :)
 * concurrent({
 *   lint: {
 *     script: 'eslint .',
 *     color: 'bgGreen.white.dim',
 *   },
 *   test: 'jest',
 *   build: {
 *     script: 'webpack'
 *   }
 * })
 * @return {string} - the command to run
 */
function concurrent(scripts) {
  var _Object$keys$reduce = Object.keys(scripts).reduce(reduceScripts, {
    colors: [],
    scripts: [],
    names: []
  }),
      colors = _Object$keys$reduce.colors,
      quotedScripts = _Object$keys$reduce.scripts,
      names = _Object$keys$reduce.names;

  var flags = ['--kill-others-on-fail', `--prefix-colors "${colors.join(',')}"`, '--prefix "[{name}]"', `--names "${names.join(',')}"`, (0, _anyShellEscape2.default)(quotedScripts)];
  var concurrently = getConcurrentlyBin();
  return `${concurrently} ${flags.join(' ')}`;

  function reduceScripts(accumulator, scriptName, index) {
    var scriptObj = scripts[scriptName];
    if (!scriptObj) {
      return accumulator;
    } else if (typeof scriptObj === 'string') {
      scriptObj = { script: scriptObj };
    }
    var _scriptObj = scriptObj,
        script = _scriptObj.script,
        _scriptObj$color = _scriptObj.color,
        color = _scriptObj$color === undefined ? defaultColors[index % defaultColors.length] : _scriptObj$color;

    if (!script) {
      return accumulator;
    }
    accumulator.names.push(scriptName);
    accumulator.colors.push(color);
    accumulator.scripts.push(script);
    return accumulator;
  }
}

/**
 * Accepts any number of nps script names, filters out
 * any falsy ones, prepends `nps` to them, and passes
 * the that to `concurrent`
 * @param {...string} scriptNames - the script names to run
 * @example
 * // will basically return `nps lint & nps "test --coverage" & nps build`
 * // but with the concurrently command and relevant flags to make
 * // it super awesome with colors and whatnot. :)
 * concurrent.nps('lint', 'test --coverage', 'build')
 * @return {string} the command to run
 */
concurrent.nps = function concurrentNPS() {
  for (var _len3 = arguments.length, scriptNames = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    scriptNames[_key3] = arguments[_key3];
  }

  return concurrent(scriptNames.map(mapNPSScripts).reduce(reduceNPSScripts, {}));

  function mapNPSScripts(scriptName, index) {
    var color = defaultColors[index];
    if (!Boolean(scriptName)) {
      return undefined;
    } else if (typeof scriptName === 'string') {
      return { script: scriptName, color };
    } else {
      return Object.assign({ color }, scriptName);
    }
  }

  function reduceNPSScripts(scripts, scriptObj) {
    if (!scriptObj) {
      return scripts;
    }
    var color = scriptObj.color,
        script = scriptObj.script;

    var _script$split = script.split(' '),
        _script$split2 = _slicedToArray(_script$split, 1),
        name = _script$split2[0];

    scripts[name] = {
      script: `nps ${quoteScript(script.trim())}`,
      color
    };
    return scripts;
  }
};

/**
 * EXPERIMENTAL: THIS DOES NOT CURRENTLY WORK FOR ALL TERMINALS
 * Takes a command and returns a version that should run in
 * another tab/window of your terminal. Currently only supports
 * Windows cmd (new window) and Terminal.app (new tab)
 * @param {string} command - the command to run in a new tab/window
 * @example
 * // returns some voodoo magic to make the terminal do what you want
 * runInNewWindow('echo hello')
 * @return {string} - the command to run
 */
function runInNewWindow(command) {
  return (0, _isWindows2.default)() ? `start cmd /k "cd ${process.cwd()} && ${command}"` : _commonTags.oneLine`
      osascript
      -e 'tell application "Terminal"'
      -e 'tell application "System Events"
      to keystroke "t" using {command down}'
      -e 'do script "cd ${process.cwd()} && ${command}" in front window'
      -e 'end tell'
    `;
}

/**
 * EXPERIMENTAL: THIS DOES NOT CURRENTLY WORK FOR ALL TERMINALS
 * Takes an nps script name and prepends it with a call to nps
 * then forwards that to `runInNewWindow` properly escaped.
 * @param {string} scriptName - the name of the nps script to run
 * @example
 * // returns a script that runs
 * // `node node_modules/.bin/nps "lint --cache"`
 * // in a new tab/window
 * runInNewWindow.nps('lint --cache')
 * @return {string} - the command to run
 */
runInNewWindow.nps = function runInNewWindowNPS(scriptName) {
  var escaped = true;
  return runInNewWindow(`node node_modules/.bin/nps ${quoteScript(scriptName, escaped)}`);
};

function quoteScript(script, escaped) {
  var quote = escaped ? '\\"' : '"';
  var shouldQuote = script.indexOf(' ') !== -1;
  return shouldQuote ? `${quote}${script}${quote}` : script;
}

function getConcurrentlyBin() {
  var concurrentlyPackagePath = require.resolve('concurrently/package.json');
  var concurrentlyDir = _path2.default.dirname(concurrentlyPackagePath);

  var _require = require(concurrentlyPackagePath),
      relativeConcurrentlyBin = _require.bin.concurrently;

  var fullConcurrently = _path2.default.join(concurrentlyDir, relativeConcurrentlyBin);
  var relativeConcurrently = _path2.default.relative(process.cwd(), fullConcurrently);
  return `node ${relativeConcurrently}`;
}

/*
  eslint
    func-name-matching:0,
    global-require:0,
    import/no-dynamic-require:0
*/